class OauthRails::OauthController < ApplicationController
  def handshake
    OAuthRails.call(:initialize_handshake, self)

    token                 = consumer.get_request_token(oauth_callback: OAuthRails.config.oauth_callback_url)
    session[:oauth_rails] = { token: token.token, secret: token.secret }

    redirect_to token.authorize_url
  end

  def authorize
    return OAuthRails.call(:initialize_authorization_failed, self) if request_token_hash.nil?

    if access_token
      OAuthRails.call(:authorization_successful, self, access_token.token, access_token.secret)
    else
      OAuthRails.call(:authorization_failed, self)
    end
  end

  private

  def consumer
    @consumer ||= ::OAuth::Consumer.new(
      OAuthRails.config.oauth_consumer_key,
      OAuthRails.config.oauth_consumer_secret,
      {
        :site               => OAuthRails.config.oauth_host,
        :request_token_path => OAuthRails.config.oauth_request_token_path,
        :authorize_path     => OAuthRails.config.oauth_authorize_path,
        :access_token_path  => OAuthRails.config.oauth_access_token_path
      }
    )
  end

  def request_token_hash
    @request_token_hash ||= session.delete(:oauth_rails)
  end

  def request_token
    @request_token ||= ::OAuth::RequestToken.new(consumer, request_token_hash['token'], request_token_hash['secret'])
  end

  def access_token
    return if @token_exchange_failed

    @access_token ||= begin
      request_token.get_access_token(oauth_verifier: params[OAuthRails.config.oauth_verifier_key])
    rescue => e
      logger.debug "OAuthRails [ERROR]: #{e}"
      @token_exchange_failed = true
      nil
    end
  end
end
