require "oauth_rails/version"
require "oauth_rails/engine"
require "oauth"

module OAuthRails
  DEFAULT_SETTINGS = {
    oauth_host: nil,
    oauth_request_token_path: '/v1/request_token',
    oauth_authorize_path: '/v1/authorize',
    oauth_access_token_path: '/v1/access_token',
    oauth_consumer_key: nil,
    oauth_consumer_secret: nil,
    oauth_callback_url: 'http://localhost:3000/oauth/authorize',
    oauth_verifier_key: :oauth_verifier,
    proc_initialize_handshake:            Proc.new { |controller_context| },
    proc_initialize_authorization_failed: Proc.new { |controller_context| OAuthRails.call(:authorization_failed, controller_context) },
    proc_authorization_successful:        Proc.new { |controller_context, token, secret| raise "Run 'OAuthRails.setup()' to set a ':proc_authorization_successful'" },
    proc_authorization_failed:            Proc.new { |controller_context| raise "Run 'OAuthRails.setup()' to set a ':proc_authorization_failed'" }
  }

  def self.setup(user_settings)
    settings = DEFAULT_SETTINGS.merge(user_settings)

    validate_settings!(settings)

    @config = ::OpenStruct.new(settings)
  end

  def self.config
    @config
  end

  def config
    return self.config if self.config

    raise "You have run OAuthRails.setup() to set the configuration!"
  end

  def self.call(name, controller_context, *args)
    name = ['proc_',name].join
    config.send(name).call(controller_context, *args)
  end

  private

  def self.validate_settings!(settings)
    settings.each do |key, value|
      raise "You have to set a value for the key ':#{key}' with 'OAuthRails.setup()'!" unless value
    end
  end
end
